#include "anti01DetectorConstruction.hh"

#include "G4PVReplica.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//

anti01DetectorConstruction::anti01DetectorConstruction()
: G4VUserDetectorConstruction()
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

anti01DetectorConstruction::~anti01DetectorConstruction()
{ }

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* anti01DetectorConstruction::Construct()
{  
  // Get nist material manager
  G4NistManager* nist = G4NistManager::Instance();
  
   
  // Option to switch on/off checking of volumes overlaps
  G4bool checkOverlaps = true;
  //     
  // Box
  //
  G4Material* box_mat = nist->FindOrBuildMaterial("G4_Pb");
  G4Material* default_mat = nist->FindOrBuildMaterial("G4_Galactic");
  G4Material* scint_mat   = nist->FindOrBuildMaterial("G4_lAr");
  G4ThreeVector posBox = G4ThreeVector(0, 0*cm, 25*cm);

  //
  // Méretek
  //
  
  G4double abs_sizeXY = 50*cm;
  G4double gap_sizeXY = 50*cm;
  G4double abs_sizeZ = 20*mm;      
  G4double gap_sizeZ = 8*mm;      

  G4int nofLayers = 32;

  G4double layerThickness = abs_sizeZ + gap_sizeZ;
  G4double boxThickness = nofLayers * layerThickness;

  //     
  // World
  //
  G4double world_sizeXY = 20*abs_sizeXY;
  G4double world_sizeZ  = 20*boxThickness;

  G4Material* world_mat = nist->FindOrBuildMaterial("G4_Galactic");
  
  G4Box* solidWorld =    
    new G4Box("World",                       //its name
       world_sizeXY/2, world_sizeXY/2, world_sizeZ/2);     //its size
      
  G4LogicalVolume* logicWorld =                         
    new G4LogicalVolume(solidWorld,          //its solid
                        world_mat,           //its material
                        "World");            //its name
                                   
  G4VPhysicalVolume* physWorld = 
    new G4PVPlacement(0,                     //no rotation
                      G4ThreeVector(),       //at (0,0,0)
                      logicWorld,            //its logical volume
                      "World",               //its name
                      0,                     //its mother  volume
                      false,                 //no boolean operation
                      0,                     //copy number
                      checkOverlaps);        //overlaps checking
                     
  
  //
  // BigBox
  //

  G4Box* solidBox =    
    new G4Box("Box",                      //its name
              abs_sizeXY/2, abs_sizeXY/2,  boxThickness/2); //its size
                
  G4LogicalVolume* logicBox =                         
    new G4LogicalVolume(solidBox,         //its solid
                        default_mat,      //its material
                        "Box");           //its name
               
  new G4PVPlacement(0,                       //no rotation
                    G4ThreeVector(),       //at (0,0,0)
                    logicBox,             //its logical volume
                    "Box",                //its name
                    logicWorld,                //its mother  volume
                    false,                   //no boolean operation
                    0,                       //copy number
                    checkOverlaps);          //overlaps checking

  //
  // layers
  //
  
  G4Box* solidLayer
    = new G4Box("Layer",           // its name
                 abs_sizeXY/2, abs_sizeXY/2, layerThickness/2); // its size
                         
  G4LogicalVolume* logicLayer
    = new G4LogicalVolume(solidLayer,           // its solid
                         default_mat,           // its material
                         "Layer");          // its name

  new G4PVReplica(
                 "Layer",          // its name
                 logicLayer,       // its logical volume
                 logicBox,         // its mother
                 kZAxis,           // axis of replication
                 nofLayers,        // number of replica
                 layerThickness);  // witdth of replica

  //
  // Absorber
  //

  G4Box* solidAbs =    
    new G4Box("Abs",                      //its name
              abs_sizeXY/2, abs_sizeXY/2,  abs_sizeZ/2); //its size
                
  G4LogicalVolume* logicAbs =                         
    new G4LogicalVolume(solidAbs,         //its solid
                        box_mat,          //its material
                        "Abs");           //its name
               
  new G4PVPlacement(0,                       //no rotation
                    G4ThreeVector(0,0,-gap_sizeZ/2),                    //at position
                    logicAbs,               //its logical volume
                    "Abs",                 //its name
                    logicLayer,               //its mother  volume
                    false,                   //no boolean operation
                    0,                       //copy number
                    checkOverlaps);          //overlaps checking
  //
  // Gap
  //

  G4Box* solidGap =    
    new G4Box("Gap",                      //its name
              gap_sizeXY/2, gap_sizeXY/2,  gap_sizeZ/2); //its size
                
  G4LogicalVolume* logicGap =                         
    new G4LogicalVolume(solidGap,         //its solid
                        scint_mat,      //its material
                        "Gap");           //its name
           
  new G4PVPlacement(0,                       //no rotation
                    G4ThreeVector(0,0,abs_sizeZ/2),                    //at position
                    logicGap,               //its logical volume
                    "Gap",                 //its name
                    logicLayer,               //its mother  volume
                    false,                   //no boolean operation
                    0,                       //copy number
                    checkOverlaps);          //overlaps checking

  logicWorld->SetVisAttributes (G4VisAttributes::GetInvisible());
  logicLayer->SetVisAttributes (G4VisAttributes::GetInvisible());

  G4VisAttributes* ColorAttr= new G4VisAttributes(G4Colour(0.0,0.0,1.0));
  ColorAttr->SetVisibility(true);
  logicAbs ->SetVisAttributes(ColorAttr);

  G4cout << *(G4Material::GetMaterialTable()) << G4endl;

  return physWorld;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
  
void anti01DetectorConstruction::ConstructSDandField()
{ 
  // Create global magnetic field messenger.
  // Uniform magnetic field is then created automatically if
  // the field value is not zero.
  G4ThreeVector fieldValue = G4ThreeVector(0.,0.,4.*tesla);
  G4GlobalMagFieldMessenger* fMagFieldMessenger = new G4GlobalMagFieldMessenger(fieldValue);
  fMagFieldMessenger->SetVerboseLevel(1);

}
